;==========================================================
ATXINITPROC ReadAllATXConfigValues
;==========================================================
; returns TRUE if all values were found in INI, FALSE otherwise
; (post-1.30 version)
push ebp
push esi
push edi
push ebx

	mov [ATXConfigComplete],TRUE
	mov esi,ATXConfigValueBuffer		; esi = string buffer (esi constant)
	mov ebx,INI_ENTRIES_TABLE		; ebx = pointer to current entry info in table

	.inientriesloop:

; get string value for next entry

	invoke GetPrivateProfileString,[ebx+ATXCONFIGENTRY.lpSectionName],[ebx+ATXCONFIGENTRY.lpKeyName],\
				       ATXCONFIG_ENTRY_ABSENT_VALUE_STR,esi,\
				       ATXCONFIG_VALUE_BUFFER_SIZE,atxconfigpath_const

	mov eax,[esi]				; eax = first characters of result
	mov ebp,TRUE				; ebp = 'exists' bool

; check for entry presence

	test al,al
	jz .doesnotexist
	cmp al,';'
	jz .doesnotexist
	cmp eax,ATXCONFIG_ENTRY_ABSENT_VALUE
	jnz .exists

		.doesnotexist:
		xor ebp,ebp
		mov [ATXConfigComplete],ebp

	.exists:

; handle string value

	call [ebx+ATXCONFIGENTRY.HandlerProc]

	add ebx,sizeof.ATXCONFIGENTRY
	cmp ebx,INI_ENTRIES_TABLE_END
	jb .inientriesloop

pop ebx
pop edi
pop esi
pop ebp
ret

;==========================================================
ATXINITPROC INI_INT_PROC
;==========================================================
; ebx = pointer to ATXCONFIGENTRY instance

	mov eax,[ebx+ATXCONFIGENTRY.DefaultValue]	; (integer)

	cmp ebp,FALSE
	jz .doesnotexist

		push [ebx+ATXCONFIGENTRY.DefaultValue]
		lea edx,[esp]
		cinvoke sscanf,esi,INT_STR,edx
		pop eax 				; eax = integer

	.doesnotexist:

; eax = integer value

	mov edx,[ebx+ATXCONFIGENTRY.lpDestVar]
	mov [edx],eax

ret

;==========================================================
ATXINITPROC INI_FLOAT_PROC
;==========================================================

	mov eax,[ebx+ATXCONFIGENTRY.DefaultValue]	; (pointer to string)

	cmp ebp,FALSE
	jz .doesnotexist

		mov eax,esi

	.doesnotexist:

; eax = pointer to string repr. of float

	push 0			; BUG: These should be default values, not always 0!!!
	lea edx,[esp]
	cinvoke sscanf,eax,FLOAT_STR,edx
	pop eax

; eax = float value

	mov edx,[ebx+ATXCONFIGENTRY.lpDestVar]
	mov [edx],eax

ret

;==========================================================
ATXINITPROC INI_STR_NOLIMIT_PROC
;==========================================================

	push esi

	cmp ebp,TRUE
	jz .exists

		mov esi,[ebx+ATXCONFIGENTRY.DefaultValue]	; (pointer to string)

	.exists:

; esi = pointer to source string

	mov edi,[ebx+ATXCONFIGENTRY.lpDestVar]
	call TransferString

	pop esi

ret

;==========================================================
ATXINITPROC INI_STR_PROC
;==========================================================

	mov BYTE [esi+ATXCONFIGVAL_MAX_STR_SIZE-1],0		; force null-terminator
	call INI_STR_NOLIMIT_PROC

ret

;==========================================================
ATXINITPROC INI_STR_EXT_PROC
;==========================================================

	mov BYTE [esi+ATXCONFIGVAL_MAX_STR_EXT_SIZE-1],0	; force null-terminator
	call INI_STR_NOLIMIT_PROC

ret

;==========================================================
ATXINITPROC INI_STR_EXTMAX_PROC
;==========================================================

	mov BYTE [esi+ATXCONFIGVAL_MAX_STR_EXTMAX_SIZE-1],0	; force null-terminator
	call INI_STR_NOLIMIT_PROC

ret

;==========================================================
ATXINITPROC INI_KEY_PROC
;==========================================================

	xor eax,eax

	cmp ebp,TRUE
	jz .exists

		mov edx,[ebx+ATXCONFIGENTRY.DefaultValue]
		mov al,[edx]
		jmp .existencechecked

	.exists:

		mov al,[esi]

	.existencechecked:

	cmp al,'0'			; ('0' = no key = usual default)
	jnz .notzero

		xor al,al
		jmp .nochar

	.notzero:

	CHAR_TOLOWERCASE al
	and eax,0FFh

	.nochar:

	mov edx,[ebx+ATXCONFIGENTRY.lpDestVar]
	mov [edx],eax

ret

;==========================================================
ATXINITPROC INI_HEX_PROC
;==========================================================

	mov eax,[ebx+ATXCONFIGENTRY.DefaultValue]	; (pointer to string)

	cmp ebp,FALSE
	jz .doesnotexist

		mov eax,esi

	.doesnotexist:

; eax = pointer to string repr. of hex value

	push 0			; BUG: These should be default values, not always 0!!!
	lea edx,[esp]
	cinvoke sscanf,eax,HEX_STR,edx
	pop eax

; eax = hex value

	mov edx,[ebx+ATXCONFIGENTRY.lpDestVar]
	mov [edx],eax

ret

;==========================================================
ATXINITPROC INI_FLOAT2X_PROC			; new in ATX 2.12 BETA
;==========================================================

	mov eax,[ebx+ATXCONFIGENTRY.DefaultValue]	; (pointer to string)

	cmp ebp,FALSE
	jz .doesnotexist

		mov eax,esi

	.doesnotexist:

; eax = pointer to string repr. of float

	push 0			; BUG: These should be default values, not always 0!!!
	push 0
	lea edx,[esp+4]
	lea ecx,[esp]
	cinvoke sscanf,eax,FLOAT2X_COLON_STR,ecx,edx
	pop ecx
	pop edx

; ecx = float value 1, edx = float value 2

	mov eax,[ebx+ATXCONFIGENTRY.lpDestVar]
	mov [eax+DWORD2X.Val1],ecx
	mov [eax+DWORD2X.Val2],edx

ret


;==========================================================
ATXINITPROC UpdateAllATXConfigINIValues
;==========================================================
; only modifies adds entries that could not be found
; 3 loops (must go around Hotkey entries - special case)
; can only be called after hotkeys have been scanned

push edi
push esi
push ebx
push ebp

; recreate ATXconfig.ini

	xor ebp,ebp

	invoke CreateFile,atxconfigpath_const,GENERIC_WRITE,0,0,CREATE_ALWAYS,\
			  FILE_ATTRIBUTE_NORMAL,0
	cmp eax,INVALID_HANDLE_VALUE
	jz .exit
	mov ebp,eax

	invoke SetFilePointer,ebp,0,0,FILE_BEGIN

; write config file header

	push 0
	mov ecx,esp
	invoke WriteFile,ebp,ATXCONFIG_HEADER,ATXCONFIG_HEADER.size,ecx,0
	test eax,eax
	pop ecx
	jz .exit
	cmp ecx,ATXCONFIG_HEADER.size
	jnz .exit


; begin value loops

	mov ebx,INI_ENTRIES_TABLE		; ebx = pointer to current entry info in table
	mov edi,ATXCONFIGHOTKEYENTRIES
	call UpdateATXConfigINIValuesSubLoop

	mov edi,ATXCONFIGHOTKEYENTRIESEND
	call UpdateATXConfigINIValuesSubLoop

; must also check for custom cheat entries (named after cheats)
	call UpdateATXConfigINIHotkeys

	mov edi,INI_ENTRIES_TABLE_END
	call UpdateATXConfigINIValuesSubLoop

; must also check for special display options
	mov eax,CAnimalObjValLocInfoTable
	mov edx,section_canimaldisplay
	call UpdateATXConfigINIObjValEnabled

.exit:

	CLOSEHANDLE ebp

pop ebp
pop ebx
pop esi
pop edi
ret

;==========================================================
ATXINITPROC UpdateATXConfigINIValuesSubLoop
;==========================================================
; ebx = ATXCONFIGENTRY instance
; edi = limit of array
; ebp = config file handle
; updates ebx

push esi					; esi = last section name pointer

	xor esi,esi

	.inientriesloop:

	cmp esi,[ebx+ATXCONFIGENTRY.lpSectionName]
	jz .notnewsection

		mov esi,[ebx+ATXCONFIGENTRY.lpSectionName]

		cinvoke wsprintf,ATXConfigValueBuffer,INI_SECTION_TEMPLATE_CRET,esi

		sub esp,4
		mov edx,esp
		invoke WriteFile,ebp,ATXConfigValueBuffer,eax,edx,0
		add esp,4

	.notnewsection:


	mov eax,[ebx+ATXCONFIGENTRY.HandlerProc]
	cmp eax,INI_INT_PROC
	jz .valueint
	cmp eax,INI_FLOAT_PROC
	jz .valuefloat
	cmp eax,INI_KEY_PROC
	jz .valuekey
	cmp eax,INI_HEX_PROC
	jz .valuehex
	cmp eax,INI_FLOAT2X_PROC
	jz .valuefloat2x

.valuestring:			; (default)

	cinvoke wsprintf,ATXConfigValueBuffer,INI_KEYVALUE_STR_TEMPLATE_CRET,\
			 [ebx+ATXCONFIGENTRY.lpKeyName],[ebx+ATXCONFIGENTRY.lpDestVar]
	jmp .valuewrite

.valueint:

	mov eax,[ebx+ATXCONFIGENTRY.lpDestVar]
	cinvoke wsprintf,ATXConfigValueBuffer,INI_KEYVALUE_INT_TEMPLATE_CRET,\
			 [ebx+ATXCONFIGENTRY.lpKeyName],DWORD [eax]
	jmp .valuewrite

.valuefloat:			; uses sprintf instead of wsprintf

	mov eax,[ebx+ATXCONFIGENTRY.lpDestVar]
	sub esp,8
	fld DWORD [eax]
	fstp QWORD [esp]

	invoke sprintf,ATXConfigValueBuffer,INI_KEYVALUE_FLOAT_TEMPLATE_CRET,\
		       [ebx+ATXCONFIGENTRY.lpKeyName]
	add esp,(4*3)+8
	jmp .valuewrite

.valuekey:

	mov edx,[ebx+ATXCONFIGENTRY.lpDestVar]
	xor eax,eax
	mov al,[edx]
	test al,al
	jnz @f

		mov edx,[ebx+ATXCONFIGENTRY.DefaultValue]
		mov al,[edx]

	@@:

	push eax
	lea edx,[esp]
	cinvoke wsprintf,ATXConfigValueBuffer,INI_KEYVALUE_STR_TEMPLATE_CRET,\
			 [ebx+ATXCONFIGENTRY.lpKeyName],edx

	add esp,4
	jmp .valuewrite

.valuehex:

	mov eax,[ebx+ATXCONFIGENTRY.lpDestVar]
	cinvoke wsprintf,ATXConfigValueBuffer,INI_KEYVALUE_HEX_TEMPLATE_CRET,\
			 [ebx+ATXCONFIGENTRY.lpKeyName],DWORD [eax]
	jmp .valuewrite

.valuefloat2x:

	mov eax,[ebx+ATXCONFIGENTRY.lpDestVar]
	sub esp,16
	fld DWORD [eax+DWORD2X.Val1]
	fstp QWORD [esp]
	fld DWORD [eax+DWORD2X.Val2]
	fstp QWORD [esp+8]

	invoke sprintf,ATXConfigValueBuffer,INI_KEYVALUE_FLOAT2X_TEMPLATE_CRET,\
		       [ebx+ATXCONFIGENTRY.lpKeyName]
	add esp,(4*3)+16

       ; jmp .valuewrite


.valuewrite:

; ATXConfigValueBuffer = text to write
; eax = size of text

	sub esp,4
	lea edx,[esp]
	invoke WriteFile,ebp,ATXConfigValueBuffer,eax,edx,0
	add esp,4


	add ebx,sizeof.ATXCONFIGENTRY
	cmp ebx,edi
	jb .inientriesloop

pop esi
ret