;====================================================================
ATXPROC AT118SwapAnimTexturePtrs			; ecx = pointer to AT118Info struct
;====================================================================
; returns 0 if error, or non-zero if success

push esi
push edi
push ebx
mov esi,ecx

	GETINSTANCEFROMID32 [esi+AT118Info.InstanceNameID]
	test eax,eax
	jz .exiterror_noinstance

	mov ecx,[eax+CInstance.lpCInfo]
	mov edi,[ecx+CInfo.lpCRenderType]		; assume CMeshAnimating

	mov edx,[edi+CMeshAnimating.NumAnimTextures]
	mov eax,[esi+AT118Info.AnimFrameNum]

	cmp edx,0
	jle .exiterror_noanimtextures

	cmp eax,edx
	jnb .exiterror_overanimtextures

	mov ecx,[edi+CMeshAnimating.lpAnimCTextureList]
	mov ebx,[ecx+eax*4]				; edx = new CTexture

	mov eax,[esi+AT118Info.SubMaterialValue]
	dec eax
	cmp eax,0
	jl .exiterror_invalidsubmaterialvalue

	cmp eax,[edi+CMesh.TextureCount]
	jnb .exiterror_overtexturecount

	mov edx,[edi+CMesh.lpTexturePointers]
	mov ecx,[edx+eax*4]				; ecx = old texture
	inc DWORD [ebx+4]				; increase new texture reference count
	mov [edx+eax*4],ebx				; replace texture pointer

	dec DWORD [ecx+4]				; decrease old texture reference count and release if necessary
	jnz @f

		mov eax,[ecx]
		call DWORD [eax+4]			; (texture release method - must call if reference count reaches zero)

	@@:

	mov eax,TRUE

pop ebx
pop edi
pop esi
ret

.exiterror_noinstance:					; (errors not currently distinguished)
.exiterror_noanimtextures:
.exiterror_overanimtextures:
.exiterror_invalidsubmaterialvalue:
.exiterror_overtexturecount:

	xor eax,eax

pop ebx
pop edi
pop esi
ret

;====================================================================
ATXPROC AT118ReapplyAllRecordedTextureChanges	; ecx = pointer to AT118Info struct
;====================================================================
push esi
push edi

	mov esi,[AT118InstanceRefBuf]
	mov edi,[AT118BufDesc.InstRefBufCut]

	test esi,esi
	jz .exitproc

	test edi,edi
	jz .exitproc

	add edi,esi

	.mainloop:
	mov ecx,esi
	call AT118SwapAnimTexturePtrs
	add esi,sizeof.AT118Info
	cmp esi,edi
	jb .mainloop

.exitproc:
pop edi
pop esi
ret

;====================================================================
ATXPROC AT118InitInstRefBuffer
;====================================================================
; Non-destructive, does nothing if buffer already exists
; AND contains valid entries (otherwise, recreates buffer).
; Called AFTER savegame presence check.

	cmp [AT118BufDesc.InstRefBufCut],0		; if cut-off for buffer is non-null, then it already contains AT118Info entries
	jnz .exitproc

	LOCALRELEASE [AT118InstanceRefBuf]		; if buffer was allocated by loading code but no entries, there may
							; have been an error; simply scrap buffer and create anew

	mov ecx,[AT118BufDesc.InstanceCount]
	test ecx,ecx
	jz .exitproc

	mov eax,sizeof.AT118Info
	mul ecx
	invoke LocalAlloc,LMEM_FIXED,eax
	test eax,eax
	jz .exitproc

	mov [AT118InstanceRefBuf],eax

.exitproc:
ret

;====================================================================
ATXPROC AT118AddToRefBuffer    ; ecx = pointer to AT118Info struct to find/replace/add
;====================================================================
push esi
push edi
push ebx

	mov edi,[AT118InstanceRefBuf]
	test edi,edi
	jz .exitproc

	mov ebx,[AT118BufDesc.InstRefBufCut]
	mov esi,ecx
	test ebx,ebx
	jz .addnewentry

	add ebx,edi

	mov eax,[esi+AT118Info.InstanceNameID]
	mov edx,[esi+AT118Info.SubMaterialValue]

	.scanloop:
	cmp [edi+AT118Info.InstanceNameID],eax
	jnz @f
		cmp [edi+AT118Info.SubMaterialValue],edx
		jz .replaceentry
	@@:
	add edi,sizeof.AT118Info
	cmp edi,ebx
	jb .scanloop

.addnewentry:		 ; edi = where to store struct

	mov eax,[esi+AT118Info.InstanceNameID]
	mov ecx,[esi+AT118Info.SubMaterialValue]
	mov edx,[esi+AT118Info.AnimFrameNum]

	mov [edi+AT118Info.InstanceNameID],eax
	mov [edi+AT118Info.SubMaterialValue],ecx
	mov [edi+AT118Info.AnimFrameNum],edx

	add [AT118BufDesc.InstRefBufCut],sizeof.AT118Info	; increment instance count (bytes in buffer)

.exitproc:
pop ebx
pop edi
pop esi
ret

.replaceentry:

	mov edx,[esi+AT118Info.AnimFrameNum]
	mov [edi+AT118Info.AnimFrameNum],edx

pop ebx
pop edi
pop esi
ret