;============================================================
MACRO ALLOCAT _size {
;============================================================
; memory allocation - returns pointer in edi or jumps to UnableToAllocateMem if error

	ccall [YAPAXI],_size			; allocates memory for instance
	test eax,eax
	jz UnableToAllocateMem

	mov edi,eax				; save pointer in edi

      }

;============================================================
MACRO ALLOCATBASE _size,_atvtable {
;============================================================
; memory allocation - returns pointer in edi and NULL in ecx or jumps to UnableToAllocateMem if error

	ALLOCAT _size

	xor ecx,ecx
	mov [edi+CATNEW.lpVTable],_atvtable
	mov [edi+CATNEW.refCount],ecx

      }

;============================================================
MACRO GETVALUE EntryValue,_number {
;============================================================
; T-Script entry fetch

	push _number				; edi and ebp must be set up properly
	push DWORD [ebp-36]			; passed value of edi
	push DWORD [ebp-32]			; passed value of esi
	mov eax,[ebp-28]			; passed value of ebx
	add eax,EntryValue			; table displacement for the T-Script value
	push eax
	call [GetEntryValueProc]
	add esp,16

      }

;============================================================
MACRO SETVALUE2 ValueType {
;============================================================
; same as SETVALUE, but only returns pointer in eax

	local .wrongvaluetype
	local .noobject

	test eax,eax
	jz .noobject
	cmp DWORD [eax+4],ValueType
	jnz .wrongvaluetype
	if (ValueType=TSCRIPT_VALUETYPE_STRING)
	  mov eax,[eax+40h]
	else
	  mov eax,[eax+28h]			; if string, this is only string length
	end if					; if other value, this is direct value
	jmp .noobject

	.wrongvaluetype:
	xor eax,eax
	.noobject:

      }

;============================================================
MACRO SETVALUEDIR ValueType,_member {
;============================================================
; retrieves and stores value from returned object
; _member is the member in the destination object

	local .wrongvaluetype
	local .noobject

	test eax,eax
	jz .noobject
	cmp DWORD [eax+4],ValueType
	jnz .wrongvaluetype
	if (ValueType=TSCRIPT_VALUETYPE_STRING)
	  mov eax,[eax+40h]
	else
	  mov eax,[eax+28h]			; if string, this is only string length (without null-terminator)
	end if					; if other value, this is direct value
	mov [edi+_member],eax
	jmp .noobject

	.wrongvaluetype:
	xor eax,eax
	.noobject:				; eax has value if needed

      }

;============================================================
MACRO SETVALUEDIRBYTE ValueType,_member {
;============================================================
; same as SETVALUEDIR, but for bools and short ints only

	local .wrongvaluetype
	local .noobject

	test eax,eax
	jz .noobject
	cmp DWORD [eax+4],ValueType
	jnz .wrongvaluetype
	mov al,[eax+28h]
	mov [edi+_member],al
	jmp .noobject

	.wrongvaluetype:
	xor eax,eax
	.noobject:

      }

;============================================================
;MACRO GETGROUP EntryValue,_number, {
;============================================================
; UNTESTED/UNUSED

;         push _number                           ; edi and ebp must be set up properly
;         push DWORD [ebp-36]                    ; passed value of edi
;         push DWORD [ebp-32]                    ; passed value of esi  -> USE CUSTOM VALUE
;         mov eax,[ebp-28]                       ; passed value of ebx
;         add eax,EntryValue                     ; table displacement for the T-Script value
;         push eax
;                                                ; param1 = address for DWORD for group ID
;         call [GetGroupIDValueProc]             ; returns al=false if not found/error
;         add esp,20

;      }

;============================================================
MACRO GETVALUEEXT EntryOffsetValue,GroupIDValue,MiscValue,CountValueAddr,ValueType {
;============================================================
; returns value in eax.
; If no error, ecx is non-null and contains structure (contains
; string length if string). If not found or if error, ecx is null.

      local .invalidentry

	push DWORD [CountValueAddr]
	push MiscValue
	push GroupIDValue
	push EntryOffsetValue
	call [GetEntryValueProc]
	xor ecx,ecx
	add esp,16

	test eax,eax
	jz .invalidentry
	cmp DWORD [eax+4],ValueType
	jnz .invalidentry
	inc DWORD [CountValueAddr]

	if (ValueType=TSCRIPT_VALUETYPE_STRING)
	  cmp DWORD [ecx+28h],0 		; if string, only return if size is longer than zero
	  jz .invalidentry
	  mov ecx,eax
	  mov eax,[ecx+40h]
	else
	  mov ecx,eax
	  mov eax,[ecx+28h]			; if string, this is only string length (without null-terminator)
	end if					; if other value, this is direct value

	.invalidentry:

      }

;============================================================
MACRO GETSETVALUERET EntryValue,_number,ValueType {
;============================================================
; higher-level macros

	GETVALUE EntryValue,_number
	SETVALUE2 ValueType

      }

;============================================================
MACRO GETSETVALUEDIR EntryValue,_number,ValueType,_member {
;============================================================

	GETVALUE EntryValue,_number
	SETVALUEDIR ValueType,_member

      }

;============================================================
MACRO GETSETVALUEDIRBYTE EntryValue,_number,ValueType,_member {
;============================================================

	GETVALUE EntryValue,_number
	SETVALUEDIRBYTE ValueType,_member

      }

;============================================================
MACRO SETLASTATERROR lpErrorString {
;============================================================
	mov [NoNewATLastAbortReason],lpErrorString
      }





