;============================================================
ATXPROC GetInGameTresDDrawObject			; returns in eax (should be NULL if none)
;============================================================
; selects D3D DirectDraw object if it exists, otherwise software

push ecx

	mov ecx,[DDRAW_OBJ_BASE]
	mov eax,[ecx+4]
	test eax,eax
	     jnz .d3dfound
	mov eax,[ecx]					; (software mode otherwise)
	.d3dfound:

pop ecx
ret

;============================================================
ATXPROC GetDDSurfaceStructSize				; returns in eax (should be NULL if none)
;============================================================
; THIS ONLY RETURNS THE RIGHT VALUE IF DRAWING OVER THE IN-GAME SCREEN (i.e., NOT IN MENUS)
; PRESERVES ecx,edx

push ecx

	mov ecx,[DDRAW_OBJ_BASE]
	mov eax,sizeof.DDSURFACEDESC2

	cmp DWORD [ecx+4],0
	    jnz .d3dfound

		 mov eax,sizeof.DDSURFACEDESC		; (software mode otherwise)

	.d3dfound:

pop ecx
ret



;=======================================================
proc DDCreateOffscreenCloneSurface uses ebx edi esi, lpOrigSurface,ResX, ResY
;=======================================================
; Creates surface with same color as given surface, but resolution may be different.
; Set ResX=ResY=0 to make same resolution
; Tries to create in video memory.
; returns new surface in eax, or null if error, and DD error code in ecx if error

local OrigSurfaceDesc:DDSURFACEDESC2, NewSurfaceDesc:DDSURFACEDESC2, lpNewSurface:DWORD

	mov eax,sizeof.DDSURFACEDESC	; force non-2 surface

	mov [OrigSurfaceDesc.dwSize],eax
	mov [NewSurfaceDesc.dwSize],eax

	lea edi,[NewSurfaceDesc.dwFlags]
	mov ecx,(sizeof.DDSURFACEDESC2 - DDSURFACEDESC2.dwFlags)
	ZEROMEM


	mov ecx,[lpOrigSurface]
	lea edx,[OrigSurfaceDesc]
	mov eax,[ecx]
	push edx
	push ecx
	call [eax+DirectDrawSurface.GetSurfaceDesc]


	mov eax,[ResX]
	mov ecx,[ResY]

	mov edx,eax
	or edx,ecx
	jnz @f

		mov eax,[OrigSurfaceDesc.dwWidth]
		mov ecx,[OrigSurfaceDesc.dwHeight]

	@@:

	mov [NewSurfaceDesc.dwWidth],eax
	mov [NewSurfaceDesc.dwHeight],ecx
	mov [NewSurfaceDesc.dwFlags],DDSD_WIDTH+DDSD_HEIGHT+DDSD_PIXELFORMAT+DDSD_CAPS
	mov [NewSurfaceDesc.ddsCaps.dwCaps],DDSCAPS_OFFSCREENPLAIN


	lea esi,[OrigSurfaceDesc.ddpfPixelFormat]
	lea edi,[NewSurfaceDesc.ddpfPixelFormat]
	mov ecx,(sizeof.DDPIXELFORMAT/4)
	rep movsd

       ; mov [NewSurfaceDesc.ddpfPixelFormat.dwSize],sizeof.DDPIXELFORMAT
       ; mov [NewSurfaceDesc.ddpfPixelFormat.dwFlags],DDPF_RGB

       ; mov eax,[OrigSurfaceDesc.ddpfPixelFormat.dwRGBBitCount]
       ; mov ecx,[OrigSurfaceDesc.ddpfPixelFormat.dwRBitMask]
       ; mov [NewSurfaceDesc.ddpfPixelFormat.dwRGBBitCount],eax
       ; mov [NewSurfaceDesc.ddpfPixelFormat.dwRBitMask],ecx

       ; mov eax,[OrigSurfaceDesc.ddpfPixelFormat.dwGBitMask]
       ; mov ecx,[OrigSurfaceDesc.ddpfPixelFormat.dwBBitMask]
       ; mov [NewSurfaceDesc.ddpfPixelFormat.dwGBitMask],eax
       ; mov [NewSurfaceDesc.ddpfPixelFormat.dwBBitMask],ecx

       ; mov [NewSurfaceDesc.ddpfPixelFormat.dwRGBAlphaBitMask],0

	GETTRESDDOBJNOD3D ebx

	lea ecx,[NewSurfaceDesc]
	lea edx,[lpNewSurface]
	mov eax,[ebx]
	push 0
	push edx
	push ecx
	push ebx
	call [eax+DirectDraw.CreateSurface]
	cmp eax,DD_OK
	mov ecx,eax
	jnz .createerror

	mov eax,[lpNewSurface]

ret

.createerror:
	xor eax,eax
ret
endp


;============================================================
ATXPROC DirectXBitmapBltWithTransparency16		; draws 16-bit bmp to screen
;============================================================
; USES TOP-LEFT PIXEL AS COLOR KEY
; params (1,2,...):
; dest. x
; dest. y
; width (of whole BMP)
; height
; pointer to bmp in memory
; use color key? set to NULL or DDBLT_KEYSRC
; image size
; flag (use 565 or 555)
; dest. RECT width
; dest. RECT height

push ebp
mov ebp,esp
push esi
push edi
push ebx

; NOTE: DirectX seems to completely refuse to create client memory surfaces, despite
; what the DirectX 6.1 documentation says. The example was tried unsuccessfully,
; along with every possible flags combinations. I debugged DDRAW.dll and found
; that when the lpSurface member was set, DirectDraw would always nullify the DDSD_PITCH
; flag, and then jump to a procedure which depended on this flag being set. Absolute
; nonsense. As a result, the bitmap data has to be 'manually' copied to a standard
; plain DDraw surface. Also, SetSurfaceDesc could not update the lpSurface member
; after creation because the method wasn't present in the DD surface interface (null?!?!?!).

	mov ebx,[ebp+8+(4*4)]

; setup DirectDraw surface for bitmap

	call GetDDSurfaceStructSize
	mov [bltclientsurface.dwSize],eax
	mov [bltclientsurface.dwFlags],DDSD_WIDTH+DDSD_HEIGHT+DDSD_PIXELFORMAT+\
				       DDSD_CKSRCBLT+DDSD_CAPS+DDSD_PITCH

	mov eax,[ebp+16]
	mov ecx,[ebp+20]
	mov [bltclientsurface.dwWidth],eax
	mov [bltclientsurface.dwHeight],ecx
	shl eax,1
	mov [bltclientsurface.lpSurface],ebx
	mov [bltclientsurface.lPitch],eax

	mov [bltclientsurface.ddpfPixelFormat.dwSize],sizeof.DDPIXELFORMAT
	mov [bltclientsurface.ddpfPixelFormat.dwFlags],DDPF_RGB
	mov [bltclientsurface.ddpfPixelFormat.dwRGBBitCount],16
	mov eax,[ebp+8+(7*4)]

	mov ecx,0000F800h
	mov edx,000007E0h

	cmp eax,SCREEN_16BITRGB_FORMAT_565
	     jz @f
	mov ecx,00007C00h
	mov edx,000003E0h
	@@:

	mov [bltclientsurface.ddpfPixelFormat.dwRBitMask],ecx
	mov [bltclientsurface.ddpfPixelFormat.dwGBitMask],edx
	mov [bltclientsurface.ddpfPixelFormat.dwBBitMask],0000001Fh

	mov [bltclientsurface.ddsCaps.dwCaps],DDSCAPS_SYSTEMMEMORY+DDSCAPS_OFFSCREENPLAIN

; get color key

	mov eax,[ebp+8+(6*4)]				; image size
	add eax,ebx					; end of image
	mov ecx,[bltclientsurface.dwWidth]
	add ecx,1
	and ecx,(not 1) 				; round up to nearest even width number (for scan line filling)
	shl ecx,2
	sub eax,ecx
	xor edx,edx
	mov dx,[eax]					; top-left pixel
	mov [bltclientsurface.ddckCKSrcBlt.dwColorSpaceLowValue],edx
	mov [bltclientsurface.ddckCKSrcBlt.dwColorSpaceHighValue],edx

; create client memory area DirectDraw surface

	call GetInGameTresDDrawObject
	test eax,eax
	     jz .exit

	push 0
	push bltclientsurfacehandle
	push bltclientsurface
	mov ecx,[eax]
	push eax
	call DWORD [ecx+DirectDraw.CreateSurface]
	test eax,eax
	    jnz .exit

; lock surface

	call GetDDSurfaceStructSize
	mov [bltclientsurfaceretinfo.dwSize],eax
	mov ecx,[bltclientsurfacehandle]
	mov [bltclientsurfaceretinfo.dwFlags],DDSD_LPSURFACE+DDSD_PITCH+DDSD_WIDTH+DDSD_HEIGHT
	mov eax,[ecx]
	push 0
	push DDLOCK_READONLY+DDLOCK_SURFACEMEMORYPTR+DDLOCK_WAIT+DDLOCK_WRITEONLY
	push bltclientsurfaceretinfo
	push 0
	push ecx
	call DWORD [eax+DirectDrawSurface.Lock]
	test eax,eax
	     jnz .exitrelease

; copy bitmap

	stdcall CopyBitmapBitsToSurface16,ebx,[bltclientsurfaceretinfo.lpSurface],\
		[bltclientsurfaceretinfo.dwWidth],[bltclientsurfaceretinfo.dwHeight],\
		[bltclientsurfaceretinfo.lPitch]

; unlock surface

	mov ecx,[bltclientsurfacehandle]
	push 0
	mov eax,[ecx]
	push ecx
	call DWORD [eax+DirectDrawSurface.Unlock]

; set RECTs

	xor eax,eax
	mov [bltclientsourcearea.left],eax
	mov [bltclientsourcearea.top],eax
	mov eax,[bltclientsurfaceretinfo.dwWidth]
	mov edx,[bltclientsurfaceretinfo.dwHeight]
	mov [bltclientsourcearea.right],eax
	mov [bltclientsourcearea.bottom],edx

	mov eax,[ebp+8]
	mov edx,[ebp+12]
	mov [bltclientdestarea.left],eax
	mov [bltclientdestarea.top],edx
	add eax,[ebp+4+(9*4)]				; dest. RECT width/height (for potential resize)
	add edx,[ebp+4+(10*4)]
	mov [bltclientdestarea.right],eax
	mov [bltclientdestarea.bottom],edx

; blit to screen

	mov [bltcientbltfx.dwSize],sizeof.DDBLTFX
	mov [bltcientbltfx.dwROP],SRCCOPY

	mov eax,[DDSURFACE_BASE_ADDR]
	mov edx,[eax]
	mov ecx,[edx+4Ch]				; get DD surface for screen

	mov eax,[ebp+8+(5*4)]				; use color key?
	or eax,DDBLT_WAIT+DDBLT_ROP
	push bltcientbltfx
	push eax
	push bltclientsourcearea
	push [bltclientsurfacehandle]
	push bltclientdestarea
	mov eax,[ecx]
	push ecx
	call DWORD [eax+DirectDrawSurface.Blt]

.exitrelease:

; release temp surface

	mov ecx,[bltclientsurfacehandle]
	mov eax,[ecx]
	push ecx
	call DWORD [eax+DirectDrawSurface.Release]

.exit:
pop ebx
pop edi
pop esi
mov esp,ebp
pop ebp
ret (10*4)
