;============================================================
ATXPROC QUEUE_Reset    ; ecx = STDQUEUEINFO, eax = max queue entries, edx = size of structure for each member
;============================================================
; set edx = eax = null if simply freeing mem
; eax = maxnodes = -1 if no practical limit

push ebx

	mov [ecx+STDQUEUEINFO.structsize],edx		; set new struct size already
	mov [ecx+STDQUEUEINFO.maxnodes],eax

	xor eax,eax
	mov ebx,[ecx+STDQUEUEINFO.newestnode]		; fetch first node
	mov [ecx+STDQUEUEINFO.nodecount],eax
	mov [ecx+STDQUEUEINFO.newestnode],eax
	mov [ecx+STDQUEUEINFO.oldestnode],eax

	test ebx,ebx
	     jz .exit

	.resetloop:
	mov ecx,ebx
	mov ebx,[ebx+STDQUEUENODE.oldernode]
	invoke LocalFree,ecx
	test ebx,ebx
	     jnz .resetloop

.exit:
pop ebx
ret

;============================================================
ATXPROC QUEUE_Add_Trunc    ; ecx = STDQUEUEINFO, edx = struct
;============================================================
; if queue is full, removes last entry first
; returns eax = non-null if success

push esi
push edi

	mov esi,ecx
	mov edi,edx

	mov eax,[esi+STDQUEUEINFO.nodecount]
	cmp eax,[esi+STDQUEUEINFO.maxnodes]
	    jb .addabsolute

		mov ecx,esi
		call QUEUE_DeleteLast			; if queue full, first remove last node

	.addabsolute:

	mov ecx,esi
	mov edx,edi
	call QUEUE_Add_Always				; same return value

pop edi
pop esi
ret

;============================================================
ATXPROC QUEUE_Add_Always    ; ecx = STDQUEUEINFO, edx = struct
;============================================================
; ignores max node count
; returns eax = null if error

push esi
push edi
push ebx

	mov esi,ecx
	mov edi,edx

	mov eax,[esi+STDQUEUEINFO.structsize]
	add eax,sizeof.STDQUEUENODE
	invoke LocalAlloc,LMEM_FIXED,eax

	test eax,eax
	     jz .error

	mov ebx,eax

	cmp [esi+STDQUEUEINFO.nodecount],0
	    jz .firstnode

	inc [esi+STDQUEUEINFO.nodecount]

	mov edx,[esi+STDQUEUEINFO.newestnode]		; will be 2nd newest node now
	mov [edx+STDQUEUENODE.newernode],ebx
	mov [ebx+STDQUEUENODE.oldernode],edx
	mov [ebx+STDQUEUENODE.newernode],0
	mov [esi+STDQUEUEINFO.newestnode],ebx

	mov ecx,[esi+STDQUEUEINFO.structsize]
	mov esi,edi
	lea edi,[ebx+sizeof.STDQUEUENODE]
	shr ecx,2
	rep movsd

	mov eax,TRUE

pop ebx
pop edi
pop esi
ret

.firstnode:

	xor eax,eax
	mov [esi+STDQUEUEINFO.nodecount],1
	mov [esi+STDQUEUEINFO.newestnode],ebx
	mov [esi+STDQUEUEINFO.oldestnode],ebx

	mov [ebx+STDQUEUENODE.newernode],eax
	mov [ebx+STDQUEUENODE.oldernode],eax

	mov ecx,[esi+STDQUEUEINFO.structsize]
	mov esi,edi
	lea edi,[ebx+sizeof.STDQUEUENODE]
	shr ecx,2
	rep movsd

	mov eax,TRUE

pop ebx
pop edi
pop esi
ret


.error:

	xor eax,eax

pop ebx
pop edi
pop esi
ret

;============================================================
ATXPROC QUEUE_DeleteLast    ; ecx = STDQUEUEINFO
;============================================================
; if empty, does nothing

push ebx
push esi

	mov ebx,[ecx+STDQUEUEINFO.oldestnode]
	mov esi,ecx

	test ebx,ebx					; if empty, do nothing
	     jz .exit

	cmp [esi+STDQUEUEINFO.nodecount],1
	    jz .lastnode

	mov eax,[ebx+STDQUEUENODE.newernode]
	mov [esi+STDQUEUEINFO.oldestnode],eax		; 2nd oldest node is now oldest
	mov DWORD [eax+STDQUEUENODE.oldernode],0	; no older node
	invoke LocalFree,ebx

	dec [esi+STDQUEUEINFO.nodecount]

.exit:

pop esi
pop ebx
ret

.lastnode:

	xor eax,eax
	mov [esi+STDQUEUEINFO.newestnode],eax
	mov [esi+STDQUEUEINFO.oldestnode],eax
	mov [esi+STDQUEUEINFO.nodecount],eax
	invoke LocalFree,ebx

pop esi
pop ebx
ret

;============================================================
ATXPROC QUEUE_FindReplaceAddStruct    ; ecx = STDQUEUEINFO, edx struct to find/replace/add (if not found), eax = size of compare (must be DWORD multiple)
;============================================================
; !!! UNTESTED
; searches for the struct data in edx, from start of struct to
; (start+eax) where eax is size of data to compare
; if found, the struct is replaced by the entire struct in edx

push esi
push edi
push ebx

	mov esi,ecx
	mov edi,edx
	mov ebx,eax

	mov edx,[esi+STDQUEUEINFO.oldestnode]
	test edx,edx
	     jz .addstruct

	.findloop:
	xor ecx,ecx

		.cmploop:
		mov eax,[ecx+edi]
		cmp [edx+sizeof.STDQUEUENODE+ecx],eax
		    jnz .endcmploop
		add ecx,4
		cmp ecx,ebx
		    jnb .replacestruct
		jmp .cmploop
		.endcmploop:

	mov edx,[edx+STDQUEUENODE.newernode]
	test edx,edx
	     jnz .findloop

.addstruct:
	mov ecx,ebx
	mov edx,edi
	call QUEUE_Add_Trunc

pop ebx
pop edi
pop esi
ret

.replacestruct:

	mov ecx,[esi+STDQUEUEINFO.structsize]
	sub ecx,ebx

	lea edi,[edi+ebx]
	lea edx,[edx+sizeof.STDQUEUENODE+ebx]
	shr ecx,2

	.copyloop:
	mov eax,[edi]
	mov [edx],eax
	add edi,4
	add edx,4
	dec ecx
	    jnz .copyloop

pop ebx
pop edi
pop esi
ret