;============================================================
ATXPROC GetLoadedBitmapDimensions
;============================================================
; ecx = buffer containing entire bitmap file, edx = pointer to dest. values (width, height, bpp)
; returns eax = 1 if OK, or null if unknown structure

	add ecx,sizeof.BITMAPFILEHEADER
	mov eax,[ecx+BITMAPINFOHEADER.biSize]

	cmp eax,sizeof.BITMAPINFOHEADER
	    jz .infoheader
	cmp eax,sizeof.BITMAPCOREHEADER
	    jz .coreheader

	xor eax,eax

ret

.infoheader:

	mov eax,[ecx+BITMAPINFOHEADER.biWidth]
	mov [edx],eax
	mov eax,[ecx+BITMAPINFOHEADER.biHeight]
	mov [edx+4],eax
	xor eax,eax
	mov ax,[ecx+BITMAPINFOHEADER.biBitCount]
	mov [edx+8],eax
	mov eax,TRUE

ret

.coreheader:

	xor eax,eax
	mov ax,[ecx+BITMAPCOREHEADER.bcWidth]
	mov [edx],eax
	mov ax,[ecx+BITMAPCOREHEADER.bcWidth]
	mov [edx+4],eax
	mov ax,[ecx+BITMAPCOREHEADER.bcBitCount]
	mov [edx+8],eax
	mov eax,TRUE

ret

;============================================================
ATXPROC LoadRGBBitmapFromFile
;============================================================
; loads a bitmap from file. Returns a BITMAPINFO structure at
; specified address and bitmap data in a VirtualAlloc buffer.
; returns eax null if error or eax = buffer if success.

; if bitmap is not BI_RGB and min 8bpp or invalid file, is not loaded.

; param1 = name of file
; param2 = pointer for BITMAPINFOHEADER structure storage (may be larger to accomodate BITMAPINFO, not affected)
; param3 = if null, no effect; if a bitcount, only loads bitmap if it matches this bitcount

push ebp
mov ebp,esp
sub esp,20
push esi
push edi
push ebx

	xor ebx,ebx
	mov [ebp-4],ebx
	mov [ebp-16],ebx

    ; open potential bitmap file

	invoke CreateFile,[ebp+8],GENERIC_READ,FILE_SHARE_READ,0,\
			  OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0
	cmp eax,-1
	mov ebx,eax
	    jz .error

	invoke GetFileSize,ebx,0
	cmp eax,-1
	mov [ebp-12],eax
	    jz .error

	cmp eax,(sizeof.BITMAPFILEHEADER+sizeof.BITMAPCOREHEADER)	;~
	mov esi,eax
	    jb .error

    ; cap header read size

	cmp esi,(sizeof.BITMAPFILEHEADER+sizeof.BITMAPINFOHEADER)
	    jb @f

		mov esi,(sizeof.BITMAPFILEHEADER+sizeof.BITMAPINFOHEADER)

	@@:

    ; temp memory for header

	invoke LocalAlloc,LMEM_FIXED,esi
	test eax,eax
	mov [ebp-4],eax
	mov edi,eax
	     jz .error

    ; read header

	invoke SetFilePointer,ebx,0,0,FILE_BEGIN
	cmp eax,-1
	    jz .error

	lea edx,[ebp-8]
	invoke ReadFile,ebx,[ebp-4],esi,edx,0
	test eax,eax
	     jz .error
	cmp esi,[ebp-8]
	    jnz .error

    ; test file header

	mov eax,[edi+BITMAPFILEHEADER.bfSize]
	mov ecx,[edi+BITMAPFILEHEADER.bfOffBits]				; ecx = offset of bitmap data in file

	cmp [edi+BITMAPFILEHEADER.bfType],'BM'
	    jnz .error
	cmp eax,[ebp-12]
	    jnz .error
	cmp ecx,eax
	    jnb .error

    ; get info/core header

	mov edx,[ebp+12]
	add edi,sizeof.BITMAPFILEHEADER
	mov eax,[edi]

	cmp eax,sizeof.BITMAPINFOHEADER
	    jz .infoheader
	cmp eax,sizeof.BITMAPCOREHEADER
	    jnz .error

    ; core header

	cmp [edi+BITMAPCOREHEADER.bcBitCount],24				; for core header, should be 24 bits for RGB
	    jnz .error

	xor eax,eax
	xor ecx,ecx
	mov ax,[edi+BITMAPCOREHEADER.bcWidth]
	mov cx,[edi+BITMAPCOREHEADER.bcHeight]
	cmp eax,0
	mov [edx+BITMAPINFOHEADER.biWidth],eax
	    jle .error
	cmp ecx,0
	mov [edx+BITMAPINFOHEADER.biHeight],ecx
	    jle .error
	mov [edx+BITMAPINFOHEADER.biBitCount],24
	jmp .infoheaderend

    ; info header

	.infoheader:

	cmp DWORD [ebp-12],(sizeof.BITMAPFILEHEADER+sizeof.BITMAPINFOHEADER)	; if info header, file size must correspond
	    jb .error
	cmp [edi+BITMAPINFOHEADER.biCompression],BI_RGB
	    jnz .error

	mov eax,[edi+BITMAPINFOHEADER.biWidth]
	mov ecx,[edi+BITMAPINFOHEADER.biHeight]
	cmp eax,0
	mov [edx+BITMAPINFOHEADER.biWidth],eax
	    jle .error
	cmp ecx,0
	mov [edx+BITMAPINFOHEADER.biHeight],ecx
	    jle .error
	mov ax,[edi+BITMAPINFOHEADER.biBitCount]
	cmp ax,8
	    jb .error
	mov [edx+BITMAPINFOHEADER.biBitCount],ax

	.infoheaderend:

     ; if necessary, re-test bitcount (param3)

	mov eax,[ebp+16]
	test eax,eax
	     jz @f

		cmp ax,[edx+BITMAPINFOHEADER.biBitCount]
		    jnz .error

	@@:

     ; finish dest. BITMAPINFOHEADER struct

       ; SizeImage is gotten from (filesize-offsetbits); cannot use depth*width*height
       ; because scan lines must be DWORD-address-aligned (when array of bits is
       ; loaded into memory at a DWORD-aligned location)

	mov eax,[ebp-12]   ;size
	sub eax,[edi-4]    ;offset to bits
	mov [edx+BITMAPINFOHEADER.biSizeImage],eax
	mov [ebp-20],eax
	mov [edx+BITMAPINFOHEADER.biSize],sizeof.BITMAPINFOHEADER

	xor eax,eax
	mov [edx+BITMAPINFOHEADER.biPlanes],1
	mov [edx+BITMAPINFOHEADER.biCompression],BI_RGB
	mov [edx+BITMAPINFOHEADER.biXPelsPerMeter],eax
	mov [edx+BITMAPINFOHEADER.biYPelsPerMeter],eax
	mov [edx+BITMAPINFOHEADER.biClrUsed],eax
	mov [edx+BITMAPINFOHEADER.biClrImportant],eax

    ; create virtual memory buffer

	invoke VirtualAlloc,0,[ebp-20],MEM_COMMIT+MEM_RESERVE,PAGE_READWRITE
	test eax,eax
	mov [ebp-16],eax
	    jz .error

    ; summon bitmap data (from file offset for bitmap data)

	invoke SetFilePointer,ebx,[edi-4],0,FILE_BEGIN
	cmp eax,-1
	    jz .error

	lea edx,[ebp-8]
	invoke ReadFile,ebx,[ebp-16],[ebp-20],edx,0
	test eax,eax
	mov ecx,[ebp-8]
	     jz .error
	cmp ecx,[ebp-20]
	    jnz .error

    ; free temp mem and release bmp file

	invoke LocalFree,[ebp-4]
	invoke CloseHandle,ebx
	mov eax,[ebp-16]

.exit:
pop ebx
pop edi
pop esi
mov esp,ebp
pop ebp
ret 12

	.error:
	VIRTUALRELEASENONULL DWORD [ebp-16]	;,DWORD [ebp-20]
	LOCALRELEASENONULL DWORD [ebp-4]
	CLOSEHANDLE ebx
	xor eax,eax
	jmp .exit

;============================================================
ATXPROC Convert24BitImageTo16Bit
;============================================================
; NOT OPTIMIZED
; param1 = pointer to source bits (must be DWORD-aligned)
; param2 = pointer to dest bits (must be DWORD-aligned)
; param3 = 24-bit width
; param4 = 24-bit height
; param5 = SCREEN_16BITRGB_FORMAT_565 or SCREEN_16BITRGB_FORMAT_555
; returns new size

push ebp
mov ebp,esp
sub esp,4
push esi
push edi
push ebx

	mov esi,[ebp+4+(1*4)]
	mov edi,[ebp+4+(2*4)]

	mov DWORD [ebp-4],Convert24BitPixelTo16Bit_565
	cmp DWORD [ebp+4+(5*4)],SCREEN_16BITRGB_FORMAT_565
	    jz @f

		mov DWORD [ebp-4],Convert24BitPixelTo16Bit_555

	@@:

	.loopheight:
	mov ebx,[ebp+4+(3*4)]					; width

		.loopwidth:
		call DWORD [ebp-4]				; convert pixel at esi
		mov [edi],ax
		add esi,3
		add edi,2
		dec ebx
		    jnz .loopwidth

       ;align pointers at DWORD boundaries for next scanline

	add esi,3						; if address has any of the two lowest bits set, will round up
	add edi,3
	and esi,(not 3)
	and edi,(not 3)
	dec DWORD [ebp+4+(4*4)] 				; height
	    jnz .loopheight

	mov eax,edi
	sub eax,[ebp+4+(2*4)]					; 16-bit image size

pop ebx
pop edi
pop esi
mov esp,ebp
pop ebp
ret 20

;============================================================
ATXPROC CopyBitmapBitsToSurface16
;============================================================
; also flips image vertically for screen draw
; 1 - bits pointer
; 2 - surface pointer
; 3 - width
; 4 - height
; 5 - pitch

push ebp
mov ebp,esp
sub esp,4
push esi
push edi
push ebx


	mov esi,[ebp+4+(1*4)]
	mov edi,[ebp+4+(2*4)]

	mov eax,[ebp+4+(5*4)]					; pitch
	mov ecx,[ebp+4+(4*4)]					; height
	mul ecx
	add edi,eax
	mov [ebp-4],edi 					; [ebp-4] points to end of surface in memory

	.loopheight:
	mov ebx,[ebp+4+(3*4)]
	mov edi,[ebp-4]
	sub edi,[ebp+4+(5*4)]
	mov [ebp-4],edi

		.loopwidth:
		mov ecx,ebx
		and ecx,(not 1)
		test ecx,ecx
		     jz .onepixel
		shr ecx,1
		rep movsd
		.onepixel:

		test ebx,1
		     jz .widtheven

			movsw

		.widtheven:

	add esi,3						; align source bits to DWORD
	and esi,(not 3)

	dec DWORD [ebp+4+(4*4)]
	    jnz .loopheight

pop ebx
pop edi
pop esi
mov esp,ebp
pop ebp
ret 20

;============================================================
ATXPROC Load8BitPaletteBitmapFromFile
;============================================================
; param1 = name of file
; param2 = pointer for BITMAPINFOHEADER structure storage (may be larger to accomodate BITMAPINFO, not affected)
; param3 = pointer for palette data
; param4 = pointer for bitmap data or NULL to return
; param5 = number of bytes between each scanline. if 0, use bitmap width
; must be 8bpp and dimensions must be power of 2 and must be at least 4x4d, uncompressed
; returns eax = bitmap data or null if failure

push ebp
mov ebp,esp
sub esp,16
push esi
push edi
push ebx

	xor ebx,ebx
	mov [ebp-16],ebx

    ; open potential bitmap file

	invoke CreateFile,[param1],GENERIC_READ,FILE_SHARE_READ,0,\
			  OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0
	cmp eax,-1
	mov ebx,eax
	    jz .error

	invoke GetFileSize,ebx,0
	cmp eax,-1
	mov [ebp-12],eax
	    jz .error

	cmp eax,(sizeof.BITMAPFILEHEADER+sizeof.BITMAPINFOHEADER+4*256+4*4)
	mov esi,eax
	    jb .error

    ; read file header only

	mov edi,stack_ext_buffer
	READFROMFILE ebx,0,sizeof.BITMAPFILEHEADER,edi,.error

    ; test file header

	mov eax,[edi+BITMAPFILEHEADER.bfSize]
	mov ecx,[edi+BITMAPFILEHEADER.bfOffBits]			; ecx = offset of bitmap data in file

	cmp [edi+BITMAPFILEHEADER.bfType],'BM'
	    jnz .error
	cmp eax,[ebp-12]
	    jnz .error
	cmp ecx,eax
	    jnb .error

	mov [ebp-8],ecx

    ; get info/core header

	mov edi,[param2]
	READFROMFILE ebx,sizeof.BITMAPFILEHEADER,sizeof.BITMAPINFOHEADER,edi,.error

    ; check info header

	mov eax,[edi+BITMAPINFOHEADER.biWidth]
	cmp eax,4
	    jl .error

	cdq
	mov ecx,2
	div ecx
	test edx,edx
	     jnz .error

	mov eax,[edi+BITMAPINFOHEADER.biHeight]
	cmp eax,4
	    jl .error

	cdq
	mov ecx,2
	div ecx
	test edx,edx
	     jnz .error

	cmp [edi+BITMAPINFOHEADER.biBitCount],8
	    jnz .error

	mov eax,[edi+BITMAPINFOHEADER.biWidth]
	mov ecx,[edi+BITMAPINFOHEADER.biHeight]
	mul ecx
	mov [edi+BITMAPINFOHEADER.biSizeImage],eax  ;set image size if not set
	add ecx,(sizeof.BITMAPFILEHEADER+sizeof.BITMAPINFOHEADER+4*256)

	cmp [ebp-12],ecx
	    jb .error

    ; read palette into buffer

	READFROMFILE ebx,sizeof.BITMAPFILEHEADER+sizeof.BITMAPINFOHEADER,\
		     (4*256),[param3],.error

    ; create virtual memory buffer if necessary

	mov eax,[param4]
	test eax,eax
	     jnz .novirtualmemalloc

	mov esi,[edi+BITMAPINFOHEADER.biSizeImage]
	mov ecx,[param5]
	test ecx,ecx
	     jz .noscanlinesizeadjust

		mov eax,[edi+BITMAPINFOHEADER.biHeight]
		mul ecx
		mov esi,eax

	.noscanlinesizeadjust:

	invoke VirtualAlloc,0,esi,MEM_COMMIT+MEM_RESERVE,PAGE_READWRITE
	test eax,eax
	    jz .error

	.novirtualmemalloc:

	mov [ebp-16],eax

    ; summon bitmap data (from file offset for bitmap data)

	cmp [param5],0
	    jz .noscanlineoffset

	invoke SetFilePointer,ebx,[ebp-8],0,FILE_BEGIN
	cmp eax,-1
	    jz .error

	mov eax,[edi+BITMAPINFOHEADER.biHeight]
	mov esi,[ebp-16]
	mov [ebp-8],eax

	.scanlineoffsetloop:
	sub esp,4
	mov ecx,esp
	invoke ReadFile,ebx,esi,[edi+BITMAPINFOHEADER.biWidth],ecx,0
	pop ecx
	test eax,eax
	     jz .error
	cmp ecx,[edi+BITMAPINFOHEADER.biWidth]
	    jnz .error

	add esi,[param5]
	dec DWORD [ebp-8]
	    jnz .scanlineoffsetloop

	jmp .exitscanlineoffset
	.noscanlineoffset:

	READFROMFILE ebx,[ebp-8],[edi+BITMAPINFOHEADER.biSizeImage],[ebp-16],.error

	.exitscanlineoffset:

    ; free temp mem and release bmp file

	invoke CloseHandle,ebx
	mov eax,[ebp-16]

.exit:
pop ebx
pop edi
pop esi
mov esp,ebp
pop ebp
ret 20

.error:

	VIRTUALRELEASENONULL DWORD [ebp-16]	;,[edi+BITMAPINFOHEADER.biSizeImage]
	CLOSEHANDLE ebx
	xor eax,eax
	jmp .exit
