;============================================================
ATXPROC CheckCharsForNull4		; eax = a DWORD of 4 chars
;============================================================
; destroys: eax,ecx
; returns in eax the same value but with all null chars converted to spaces

	mov ecx,4
	.scanloop:
	test al,al
	     jnz @f
	     mov al,20h
	@@:
	rol eax,8
	dec ecx
	    jnz .scanloop

ret

;============================================================
ATXPROC CompareStrings			; ecx = string1, edx = string2
;============================================================
; ignores case
; returns FALSE if not same, edx and ecx point to null-terminators

	.compareloop:			; edi points to dest.'s null-terminator (BYTE [edi] = 0)
	mov al,[ecx]
	mov ah,[edx]			; transfer/append string
	CHAR_TOLOWERCASE al
	CHAR_TOLOWERCASE ah
	cmp ah,al
	    jnz .stringsnotequal
	test al,al
	     jz .stringsequal
	inc ecx
	inc edx
	jmp .compareloop

.stringsequal:

	mov eax,TRUE

ret

.stringsnotequal:

	xor eax,eax

ret

;============================================================
ATXPROC TransferString			; esi = source, edi = dest., destroyed: esi,edi, al
;============================================================

	.transferloop:			; edi points to dest.'s null-terminator (BYTE [edi] = 0)
	mov al,[esi]
	mov [edi],al			; transfer/append string
	test al,al
	     jz .exitloop
	inc esi
	inc edi
	jmp .transferloop

.exitloop:
ret

;============================================================
ATXPROC FindWORDInString		; eax = string (null-terminated), dx = WORD to find
;============================================================
; returns -1 or address of WORD value in string (not offset)

	cmp BYTE [eax],0
	    jz .notfound

	.scanloop:
	mov cx,[eax]
	test ch,ch
	     jz .notfound
	cmp cx,dx
	    jz .found
	inc eax
	jmp .scanloop

.found:
ret

.notfound:

	mov eax,-1

ret

;============================================================
ATXPROC TransferStringToUpper		; esi = source, edi = dest., destroyed: esi,edi, al
;============================================================

	.transferloop:			; edi points to dest.'s null-terminator (BYTE [edi] = 0)
	mov al,[esi]

	cmp al,'a'
	    jb .noupper
	cmp al,'z'
	    ja .noupper

		sub al,('a'-'A')

	.noupper:

	mov [edi],al			; transfer/append string

	test al,al
	     jz .exitloop

	inc esi
	inc edi
	jmp .transferloop

.exitloop:
ret





;===================================================================
; TYPE FORMATTING FUNCTIONS
;===================================================================

; Input for each of the following functions:
; eax = pointer to value of proper data type
; edx = buffer for output string
; return value: length of string (without null-terminating char)

;===================================================================
ATXPROC TypeToString
;===================================================================
; ecx = one of all possible ATXVALTYPE_ constants

	cmp ecx,((.jumptableend-.jumptable)/4)
	jnb .paramerror 	; ATXVALTYPE_NONSTANDARD is ignored

	jmp dword [.jumptable+ecx*4]

.jumptable:
	dd	TypeToStringBoolByte		 ; ATXVALTYPE_BOOLBYTE             equ     0
	dd	TypeToStringInt32		 ; ATXVALTYPE_INT32                equ     1
	dd	TypeToStringFloat32		 ; ATXVALTYPE_FLOAT32              equ     2
	dd	TypeToStringString		 ; ATXVALTYPE_STRING               equ     3
	dd	TypeToStringAddress		 ; ATXVALTYPE_ADDRESS              equ     4
	dd	TypeToStringVector		 ; ATXVALTYPE_VECTOR               equ     5
	dd	TypeToStringQuaternion		 ; ATXVALTYPE_QUATERNION           equ     6
	dd	TypeToStringAddressImm		 ; ATXVALTYPE_ADDRESSIMM           equ     7
.jumptableend:

.paramerror:
	xor eax,eax

ret

;===================================================================
ATXPROC TypeToStringBoolByte
;===================================================================
	cmp byte [eax],0
	jz @f

	mov eax,4
	mov dword [edx],"True"
	mov byte [edx+eax],0
ret
@@:
	mov dword [edx],"Fals"
	mov dword [edx+4],"e"
	mov eax,5
ret

;===================================================================
ATXPROC TypeToStringInt32
;===================================================================
	cinvoke wsprintf,edx,INT_STR,[eax]
ret

;===================================================================
ATXPROC TypeToStringFloat32
;===================================================================
	sub esp,8
	fld DWORD [eax]
	fstp QWORD [esp]
	invoke sprintf,edx,FLOAT_PRECISE_STR
	add esp,16
ret


;===================================================================
ATXPROC TypeToStringString
;===================================================================
	push esi
	push edi
	mov esi,eax
	mov edi,edx
	push edx
	call TransferString
	pop edx
	mov eax,edi
	sub eax,edx
	pop edi
	pop esi
ret

;===================================================================
ATXPROC TypeToStringAddress
;===================================================================
	cinvoke wsprintf,edx,HEX_MARKED_STR,[eax]
ret

;===================================================================
ATXPROC TypeToStringVector
;===================================================================
	sub esp,24
	fld DWORD [eax]
	fstp QWORD [esp]
	fld DWORD [eax+4]
	fstp QWORD [esp+8]
	fld DWORD [eax+8h]
	fstp QWORD [esp+16]

	invoke sprintf,edx,FLOAT_VECTOR_STR
	add esp,32
ret

;===================================================================
ATXPROC TypeToStringQuaternion
;===================================================================
	sub esp,32
	fld DWORD [eax]
	fstp QWORD [esp]
	fld DWORD [eax+4h]
	fstp QWORD [esp+8]
	fld DWORD [eax+8h]
	fstp QWORD [esp+16]
	fld DWORD [eax+0Ch]
	fstp QWORD [esp+24]

	invoke sprintf,edx,FLOAT_QUATERNION_STR
	add esp,40
ret


;===================================================================
ATXPROC TypeToStringAddressImm
;===================================================================
	cinvoke wsprintf,edx,HEX_MARKED_STR,eax
ret

