; Compile using FASM. Version used for last release: 1.68 (Aug 24, 2009)
;
; Updates:
; 2010-02-08:
;  - Added /q switch for quiet mode (no prompts or dialogs if successful)
;

INCLUDEGENERATOR	 equ	 '1.1'

format PE GUI 4.0
entry START
stack 20000h

include 'WIN32A.INC'	; must have FASM Include environment variable set properly for this

MAX_PATH	equ	260

;=================================================================
section '.code' code readable executable
;=================================================================

include 'list.asm'


START:
push ebp
push ebx
push edi
push esi

invoke GetCommandLine
       test eax,eax
       jz .nocmdline

       call SkipFirstCmdParameter
       test eax,eax
       jz .nocmdline


       .cmdloop:

	   call GetNextCmdParameter
	   test eax,eax
	   jz .nocmdline

	   cmp cl,'q'
	   jz .quietmodeparam
	   cmp cl,'Q'
	   jnz .cmdloop

       .quietmodeparam:

	   mov [QuietModeEnabled],TRUE

       .nocmdline:

       cmp [QuietModeEnabled],TRUE
       jz .skipconfirm

; confirm!
invoke MessageBox,0,introtext,msgboxtitle,MB_YESNO
       cmp eax,IDYES
	   jnz .exitnormal

       .skipconfirm:


; get process variables

invoke GetModuleHandle,NULL
       test eax,eax
	    jz .modulehandleerror

invoke GetModuleFileName,NULL,basefolderpath,MAX_PATH
       test eax,eax
	    jz .modulehandleerror

invoke GetProcessHeap
       test eax,eax
	    jz .getheaperror
       mov [searchhandles.heapHandle],eax

; set appropriate start paths
       mov edi,basefolderpath
       call FindFilePart
       mov BYTE [eax],0
       mov [basefolderfpart],eax
       sub eax,basefolderpath

       add eax,currentfolderpath
       mov [currentfolderbase],eax
       mov [currentfolderfpart],eax

       mov esi,basefolderpath
       mov edi,currentfolderpath
       call TransferString

; delete all *.inc files in base folder

       mov esi,incsearchstr
       mov edi,[basefolderfpart]
       call TransferString

       invoke FindFirstFile,basefolderpath,finddatainc
       cmp eax,INVALID_HANDLE_VALUE
	   jz .endincdelete
       mov ebx,eax

       .deletenextinc:
       mov eax,[finddata.dwFileAttributes]
       test eax,FILE_ATTRIBUTE_HIDDEN+FILE_ATTRIBUTE_SYSTEM+\
		FILE_ATTRIBUTE_TEMPORARY+FILE_ATTRIBUTE_DIRECTORY+FILE_ATTRIBUTE_READONLY
	    jnz .checknextinctodelete

       mov esi,finddatainc.cFileName
       mov edi,[basefolderfpart]
       call TransferString
       invoke DeleteFile,basefolderpath

       .checknextinctodelete:
       invoke FindNextFile,ebx,finddatainc
       test eax,eax
	    jnz .deletenextinc

       invoke FindClose,ebx
       .endincdelete:

; start with initial base folder (do not perform include for base folder)

       mov esi,foldersearchstr
       mov edi,[currentfolderfpart]
       call TransferString
       invoke FindFirstFile,currentfolderpath,finddata
       cmp eax,INVALID_HANDLE_VALUE
	   jz .exitsuccess	   ; no sub-folders?
       mov ebx,eax
       jmp .trysubfolder

 ; ---------- main search loop

       .folderloop:
       inc [numsubfolders]
       call createIncludeDataForFolder	; create includes for this folder

       mov esi,foldersearchstr
       mov edi,[currentfolderfpart]
       call TransferString
       invoke FindFirstFile,currentfolderpath,finddata	  ; search for first sub-folder
       cmp eax,INVALID_HANDLE_VALUE
	   jz .nosubfolders				  ; no more sub-folders?
       mov ebx,eax

       .trysubfolder:
       mov eax,[finddata.dwFileAttributes]
       test eax,FILE_ATTRIBUTE_HIDDEN+FILE_ATTRIBUTE_SYSTEM+FILE_ATTRIBUTE_TEMPORARY
	    jnz .checkfornextsubfolder
       test eax,FILE_ATTRIBUTE_DIRECTORY
	    jz .checkfornextsubfolder
       cmp BYTE [finddata.cFileName],'.'
	   jz .checkfornextsubfolder

	    ; we have a sub-folder to 'go into'
	    mov eax,ebx
	    mov ecx,searchhandles
	    call addLastToList		     ; save current handle to list

	    mov esi,finddata.cFileName
	    mov edi,[currentfolderfpart]
	    call TransferString 	     ; and 'go into' sub-folder
	    mov BYTE [edi],'\'
	    inc edi
	    mov [currentfolderfpart],edi
	    jmp .folderloop

       .checkfornextsubfolder:
       invoke FindNextFile,ebx,finddata
       test eax,eax
	    jnz .trysubfolder

       invoke FindClose,ebx

       .nosubfolders:
       cmp [searchhandles.numNodes],0
	   jz .exitsuccess	      ; have we come back to base folder?

       mov eax,[currentfolderfpart]
       mov edx,[currentfolderbase]
       dec eax
       call FindFilePartFromEnd      ; go back one folder
       mov [currentfolderfpart],eax

       mov ecx,searchhandles
       call removeLastFromList	     ; get last folder's search handle in eax
       mov ebx,eax
       jmp .checkfornextsubfolder

 ; ----------

.exitsuccess:
cinvoke wsprintf,temp_buf,exittext,[numsubfolders],[numincfiles],[numincludes]

       cmp [QuietModeEnabled],TRUE
       jz .nosuccessdialog

invoke MessageBox,0,temp_buf,msgboxtitle,MB_OK

	.nosuccessdialog:


       xor edi,edi
.exitnormal:
       mov ecx,searchhandles
       call freeList

       mov eax,edi
       pop esi
       pop edi
       pop ebx
       pop ebp
invoke ExitProcess,eax

.mainerror:
invoke MessageBox,0,eax,msgboxtitle,0
       mov edi,1
       jmp .exitnormal

.modulehandleerror:
       mov eax,errornohandle
jmp .mainerror

.modulefilenameerror:
       mov eax,errornobasepath
jmp .mainerror

.getheaperror:
       mov eax,errornoheap
jmp .mainerror


;============================================================
TransferString: 	;esi=source, edi=dest., destroyed: esi,edi, al
;============================================================
TransferStringLoop: ;edi points to dest.'s null-terminator (BYTE [edi] = 0)
     mov al,[esi]
     mov [edi],al	      ;transfer/append string
     test al,al
	  jz End_TransferStringLoop
     inc esi
     inc edi
     jmp TransferStringLoop
End_TransferStringLoop:
ret


;============================================================
FindFilePart:	  ; edi = string
;============================================================

	xor ecx,ecx
	mov eax,edi

	.scanloop:
	cmp [edi],cl
	    jz .exit
	cmp BYTE [edi],'\'
	    jnz @f
	    lea eax,[edi+1]
	@@:
	inc edi
	jmp .scanloop

.exit:
ret

;============================================================
FindFilePartFromEnd:
;============================================================
; eax = current address
; edx = beginning of string

	cmp eax,edx
	    jna .exit

	.scanloop:
	cmp BYTE [eax-1],'\'
	    jz .exit
	dec eax
	cmp eax,edx
	    ja .scanloop

.exit:
ret

;============================================================
createIncludeDataForFolder:
;============================================================
; file info must be in finddata
; uses basefolderpath, basefolderfpart, baseffpathoffset, currentfolderpath,
; currentfolderfpart
push esi
push edi
push ebx

	mov esi,incsearchstr
	mov edi,[currentfolderfpart]
	call TransferString

	invoke FindFirstFile,currentfolderpath,finddatainc
	cmp eax,INVALID_HANDLE_VALUE
	    jz .exit
	mov ebx,eax

	.processfile:
	mov eax,[finddatainc.dwFileAttributes]
	test eax,FILE_ATTRIBUTE_HIDDEN+FILE_ATTRIBUTE_SYSTEM+FILE_ATTRIBUTE_TEMPORARY+FILE_ATTRIBUTE_DIRECTORY
	     jnz .trynextfile

    ; write include directive
	inc [numincfiles]	 ; increase number of write attempts

	mov esi,finddatainc.cFileName
	mov edi,[basefolderfpart]
	call TransferString

	invoke CreateFile,basefolderpath,GENERIC_WRITE,FILE_SHARE_READ,\
			  0,OPEN_ALWAYS,FILE_ATTRIBUTE_NORMAL,0
	cmp eax,INVALID_HANDLE_VALUE
	    jz .exitfilewrite1
	mov esi,eax

	invoke SetFilePointer,esi,0,0,FILE_END
	cmp eax,-1
	    jz .exitfilewrite2

	mov edx,[currentfolderfpart]
	mov BYTE [edx],0
	cinvoke wsprintf,temp_buf,includetemplate,[currentfolderbase],\
			finddatainc.cFileName
	push eax
	invoke WriteFile,esi,temp_buf,eax,tempdd,0
	test eax,eax
	pop ecx
	     jz .exitfilewrite2
	cmp ecx,[tempdd]
	    jnz .exitfilewrite2
	inc [numincludes]	; only add to count if success
	.exitfilewrite2:
	invoke CloseHandle,esi
	.exitfilewrite1:

	.trynextfile:
	invoke FindNextFile,ebx,finddatainc
	test eax,eax
	     jnz .processfile

	invoke FindClose,ebx

.exit:
pop ebx
pop edi
pop esi
ret


;============================================================
SkipFirstCmdParameter:	  ; eax = pointer to advance
;============================================================

	test eax,eax
	jz .notfound

	mov cl,[eax]
	test cl,cl
	jz .notfound

	inc eax

	mov cl,[eax]
	test cl,cl
	jz .notfound

	cmp cl,'"'
	jz @f
	    mov cl,' '
	@@:

	mov dl,[eax]
	.scanloop:

	    test dl,dl
	    jz .notfound
	    inc eax
	    cmp dl,cl
	    mov dl,[eax]
	jnz .scanloop

	cmp dl,'"'
	jnz @f
	    inc eax
	@@:

ret

.notfound:

	xor eax,eax

ret


;============================================================
GetNextCmdParameter:	; eax = pointer to advance
;============================================================
; returns cl = switch character and advances eax past pointer

	test eax,eax
	jz .notfound


	.scanloop:

	mov dl,[eax]
	test dl,dl
	jz .notfound

	inc eax
	cmp dl,'/'
	jnz .scanloop

	mov cl,[eax]
	test cl,cl
	jz .notfound

	inc eax


ret

.notfound:

	xor eax,eax

ret


;=================================================================
section '.data' data readable writeable
;=================================================================
temp_buf	     db      2000 dup 0

msgboxtitle	     db      "FASM Same-Name File Include Generator",0
introtext	     db      "WARNING: This program deletes all *.inc files found in the same directory",0Dh,0Ah
		     db      "as its executable resides in and is being run from.",0Dh,0Ah
		     db      0Dh,0Ah
		     db      "Are you sure you want to generate include files in the current directory",0Dh,0Ah
		     db      "with names of all *.inc files found in its sub-folders? ",0Dh,0Ah
		     db      0Dh,0Ah
		     db      "The current folder will contain one *.inc file for every different source file name found.",0Dh,0Ah
		     db      "Each of these files will in turn contain relative include directives for all files with the",0Dh,0Ah
		     db      "same name found in sub-folders of this directory."
		     db      0

errornohandle	     db      "Error: could not retrieve module handle.",0
errornobasepath      db      "Error: could not retrieve current path.",0
errornoheap	     db      "Error: could not retrieve program heap.",0

exittext	     db      "Include file generation completed successfully.",0Dh,0Ah
		     db      0Dh,0Ah
		     db      "Number of sub-folders traversed: %u",0Dh,0Ah
		     db      "Number of *.inc files found: %u",0Dh,0Ah
		     db      "Number of include directives generated: %u",0Dh,0Ah
		     db      0Dh,0Ah
		     db      "If the two last statistics are not equal, the program was",0Dh,0Ah
		     db      "unable to write include directives for all source files found.",0Dh,0Ah
		     db      "This can be caused by read-only, system, temporary, or in-use files."
		     db      0

foldersearchstr      db      "*",0
incsearchstr	     db      "*.inc",0
includetemplate      db      0Dh,0Ah,"include '%s%s'"
		     db      0

basefolderpath	     db      MAX_PATH dup 0
basefolderfpart      dd      0
currentfolderpath    db      MAX_PATH dup 0
currentfolderfpart   dd      0
currentfolderbase    dd      0

numsubfolders	     dd      0
numincfiles	     dd      0
numincludes	     dd      0

searchhandles	     LIST
finddata	     WIN32_FIND_DATA
finddatainc	     WIN32_FIND_DATA

tempdd		     dd      0
QuietModeEnabled     db      FALSE

;=================================================================
section '.idata' import data readable writeable
;=================================================================
  library kernel,'KERNEL32.DLL',\
	  user,'USER32.DLL'

  import kernel,\
	 CloseHandle,"CloseHandle",\
	 CreateFile,"CreateFileA",\
	 DeleteFile,"DeleteFileA",\
	 ExitProcess,"ExitProcess",\
	 FindClose,'FindClose',\
	 FindFirstFile,'FindFirstFileA',\
	 FindNextFile,'FindNextFileA',\
	 GetCommandLine,'GetCommandLineA',\
	 GetFileSize,"GetFileSize",\
	 GetModuleFileName,'GetModuleFileNameA',\
	 GetModuleHandle,'GetModuleHandleA',\
	 GetProcessHeap,'GetProcessHeap',\
	 HeapAlloc,'HeapAlloc',\
	 HeapFree,'HeapFree',\
	 SetFilePointer,'SetFilePointer',\
	 WriteFile,'WriteFile'

  import user,\
	 MessageBox,'MessageBoxA',\
	 wsprintf,"wsprintfA"

;=================================================================
section '.rsrc' resource data readable
;=================================================================

	directory RT_VERSION,versions

	resource versions,\
	   1,LANG_NEUTRAL,version

	versioninfo version,VOS__WINDOWS32,VFT_APP,VFT2_UNKNOWN,\
				LANG_ENGLISH+SUBLANG_DEFAULT,0,\
	  'FileDescription','FASM same-name file include generator.',\
	  'LegalCopyright','Free, open source.',\
	  'FileVersion',INCLUDEGENERATOR,\
	  'ProductVersion',INCLUDEGENERATOR,\
	  'OriginalFilename','INCLUDEGENERATOR.exe'

